package com.unis.ynls.dataDictionary.controller;

import com.unis.common.utils.CurrentUtil;
import com.unis.ynls.dataDictionary.entity.DataDictionaryDO;
import com.unis.ynls.dataDictionary.service.DataDictionaryGroupService;
import com.unis.ynls.dataDictionary.service.DataDictionaryService;
import com.unis.ynls.dataDictionary.utils.ResponseWrapper;
import com.unis.ynls.dataDictionary.vo.DataDictionaryGroupVO;
import com.unis.ynls.dataDictionary.vo.DataDictionaryVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.ObjectUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 *
 * @ClassName: DataDictionaryController
 * @Description: 数据字典管理控制层类
 * @author 杨祖榕
 * @date 2018年11月09日 下午20:08:23
 * @version v1.0
 */
@RestController
@RequestMapping("/dataDictionary")
@Slf4j
@Api(value = "数据字典控制层类", tags = "数据字典控制层类")
public class DataDictionaryController {

    @Autowired
    private DataDictionaryService dataDictionaryService;

    @Autowired
    private DataDictionaryGroupService dataDictionaryGroupService;

    /**
     * 根据id查询数据字典
     * @param id
     * @return ResponseWrapper
     * @author 杨祖榕
     * @date 2018年11月10日 上午10:50:23
     * @version v1.0
     */
    @ApiOperation(value = "根据id查询数据字典", notes = "参数：id")
    @GetMapping(value = "/findById/{id}")
    public ResponseWrapper findById(@PathVariable("id") Integer id) {
        try {
            DataDictionaryVO dataDictionaryVO = dataDictionaryService.findById(id);
            if (ObjectUtils.equals(dataDictionaryVO, null)) {
                return ResponseWrapper.success(300, "数据字典数据不存在");
            }
            return ResponseWrapper.success(200, "查询成功", dataDictionaryVO);
        } catch (Exception e) {
            log.error("查询数据字典数据，发生异常", e.getMessage());
            return ResponseWrapper.error(500, "数据字典数据查询失败");
        }
    }


    /**
     * 根据id查询数据字典
     * @param groupId
     * @return ResponseWrapper
     * @author 杨祖榕
     * @date 2018年11月10日 上午10:50:23
     * @version v1.0
     */
    @ApiOperation(value = "根据分组id查询数据字典", notes = "参数：groupId")
    @GetMapping(value = "/findByGroupId/{groupId}")
    public ResponseWrapper findByGroupId(@PathVariable("groupId") String groupId) {
        try {
            List<DataDictionaryDO> dataDictionaryVO = dataDictionaryService.findByGroupId(groupId);
            if (ObjectUtils.equals(dataDictionaryVO, null)) {
                return ResponseWrapper.success(300, "数据字典数据不存在");
            }
            return ResponseWrapper.success(200, "查询成功", dataDictionaryVO);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("查询数据字典数据，发生异常", e.getMessage());
            return ResponseWrapper.error(500, "数据字典数据查询失败");
        }
    }

    /**
     *根据父级id查询数据字典
     * @param pdictCode
     * @return
     */
    @ApiOperation(value = "新版根据父级id查询数据字典", notes = "参数：groupId")
    @GetMapping(value = "/findByPDictCode/{pdictCode}")
    public ResponseWrapper findByPDictCode(@PathVariable("pdictCode") String pdictCode) {
        try {
            List<DataDictionaryDO> dataDictionaryVO = dataDictionaryService.findByPDictCode(pdictCode);
            if (ObjectUtils.equals(dataDictionaryVO, null)) {
                return ResponseWrapper.success(300, "数据字典数据不存在");
            }
            return ResponseWrapper.success(200, "查询成功", dataDictionaryVO);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("查询数据字典数据，发生异常", e.getMessage());
            return ResponseWrapper.error(500, "数据字典数据查询失败");
        }
    }



    @ApiOperation(value = "新版根据groupId和dictCode查询数据字典", notes = "参数：groupId，dictCode")
    @GetMapping(value = "getDataDictionary/{groupId}/{dictCode}")
    public ResponseWrapper getDataDictionaryByGroupIdAndDictCode(@PathVariable("groupId") String groupId, @PathVariable("dictCode") String dictCode){

        try {
            DataDictionaryVO dataDictionaryVO = dataDictionaryService.getDataDictionaryByGroupIdAndDictCode(groupId, dictCode);
            if (ObjectUtils.equals(dataDictionaryVO, null)) {
                return ResponseWrapper.success(300, "数据字典数据不存在");
            }
            return ResponseWrapper.success(200, "查询成功", dataDictionaryVO);
        } catch (Exception e) {
            log.error("查询数据字典数据，发生异常", e.getMessage());
            return ResponseWrapper.error(500, "数据字典数据查询失败");
        }

    }

    /**
     * 根据数据字典组ID和数据字典名称查询数据字典
     * @param groupId
     * @param dictName
     * @return ResponseWrapper
     * @author liyb
     * @date 2019-03-19 11:02:10
     * @version v1.0
     */
    @ApiOperation(value = "新版根据数据字典组ID和数据字典名称查询数据字典", notes = "参数：数据字典组ID--groupId，数据字典名称--dictName")
    @GetMapping(value = "getDataDictionaryCode/{groupId}/{dictName}")
    public ResponseWrapper getDataDictionaryByGroupIdAndDictName(@PathVariable("groupId") String groupId, @PathVariable("dictName") String dictName){
        try {
            String dictCode = dataDictionaryService.getDataDictionaryByGroupIdAndDictName(groupId, dictName);
            return ResponseWrapper.success(200, "查询成功", dictCode);
        } catch (Exception e) {
            log.error("查询数据字典数据，发生异常", e.getMessage());
            return ResponseWrapper.error(500, "数据字典数据查询失败");
        }

    }

    /**
     * 根据id 删除数据字典数据
     * @param id
     * @return ResponseWrapper
     * @author 杨祖榕
     * @date 2018年11月10日 上午10:50:23
     * @version v1.0
     */
    @ApiOperation(value = "新版删除数据字典", notes = "参数：数据字典ID")
    @DeleteMapping(value = "/delete/{id}")
    public ResponseWrapper delete(@PathVariable("id") Integer id) {
        try {
            Boolean flag = dataDictionaryService.delete(id);
            if (! flag) {
                return ResponseWrapper.error(300, "参数错误，删除失败");
            }
            return ResponseWrapper.success(200, "数据字典数据删除成功");
        } catch (Exception e) {
            log.error("删除数据字典数据，发生异常", e.getMessage());
            return ResponseWrapper.error(500, "数据字典数据删除失败");
        }
    }

    /**
     * 保存数据字典
     * @param dataDictionaryVO
     * @return ResponseWrapper
     * @author 杨祖榕
     * @date 2018年11月10日 上午10:50:23
     * @version v1.0
     */
    @ApiOperation(value = "新版保存数据字典", notes = "参数：数据字典")
    @PostMapping(value = "/save")
    public ResponseWrapper save(@RequestBody DataDictionaryVO dataDictionaryVO) {
        try {
            List<DataDictionaryVO> dictionaryVOS = dataDictionaryService.getSubDataDictionarysByGroupId(dataDictionaryVO.getGroupId());
            for (DataDictionaryVO vo:dictionaryVOS) {
                if (dataDictionaryVO.getDictName().equals(vo.getDictName())){
                    return ResponseWrapper.error("该字典名称已存在");
                }
                if (dataDictionaryVO.getDictCode().equals(vo.getDictCode())){
                    return ResponseWrapper.error("该字典编码已存在");
                }
            }
            boolean flag = true;
            if (dictionaryVOS==null){
                 flag = dataDictionaryService.save(dataDictionaryVO);
            }else {
                flag = dataDictionaryService.save(dataDictionaryVO);
            }
            if (flag) {
                return ResponseWrapper.success(200, "保存成功");
            }
            return ResponseWrapper.error();
        } catch (Exception e) {
            log.error("保存数据字典，发生异常", e.getMessage());
            return ResponseWrapper.error(500, "保存失败");
        }
    }

    /**
     * 更新数据字典
     * @param dataDictionaryVO
     * @return ResponseWrapper
     * @author 杨祖榕
     * @date 2018年11月10日 下午16:15:23
     * @version v1.0
     */
    @ApiOperation(value = "新版更新数据字典信息", notes = "参数：数据字典")
    @PostMapping(value = "/update")
    public ResponseWrapper update(@RequestBody DataDictionaryVO dataDictionaryVO) {
        try {
            List<DataDictionaryVO> dictionaryVOS = dataDictionaryService.getSubDataDictionarysByGroupId(dataDictionaryVO.getGroupId());
            for (DataDictionaryVO vo:dictionaryVOS) {
                if (dataDictionaryVO.getDictName().equals(vo.getDictName())){
                    return ResponseWrapper.error("该字典名称已存在");
                }
            }
            boolean flag = dataDictionaryService.update(dataDictionaryVO);
            if (flag) {
                return ResponseWrapper.success(200,"修改成功");
            }
            return ResponseWrapper.error();
        } catch (Exception e) {
            log.error("更新数据字典数据，发生异常", e.getMessage());
            return ResponseWrapper.error(500, "操作失败");
        }
    }

    /**
     * 获取某dictCode下的子数据字典(非递归)
     * @param dictCode
     * @return
     * @author 杨祖榕
     * @date 2018年11月10日 下午16:15:23
     * @version v1.0
     */
    @ApiOperation(value="获取子数据字典(非递归)", notes = "获取子数据字典(非递归)")
    @GetMapping(value = "/getSubDataDictionarysByDictCodeId/{dictCode}")
    public ResponseWrapper getSubDataDictionarysByDictCodeId(@PathVariable("dictCode") String dictCode){
        try {
            dataDictionaryService.dataDictionarys2Redis();
            List<DataDictionaryVO> list = dataDictionaryService.getSubDataDictionarysByDictCodeId(dictCode);
            if (ObjectUtils.notEqual(list, null)) {
                return ResponseWrapper.success( 200, "查询成功", list );
            }
            return ResponseWrapper.error();
        } catch (Exception e) {
            log.error("数据字典查询失败{}", e.getMessage());
            return ResponseWrapper.error(500, "数据字典查询失败");
        }
    }

    /**
     * 获取某字典组ID下的第一层数据字典(非递归)
     * @param groupId
     * @return
     * @author 杨祖榕
     * @date 2018年11月10日 下午16:15:23
     * @version v1.0
     */
    @ApiOperation(value="获取某字典组ID下的第一层数据字典(非递归)", notes = "获取某字典组ID下的第一层数据字典(非递归)")
    @GetMapping(value = "/getSubDataDictionarysByGroupId/{groupId}")
    public ResponseWrapper getSubDataDictionarysByGroupId(@PathVariable("groupId") String groupId){
        try {
            List<DataDictionaryVO> list = dataDictionaryService.getSubDataDictionarysByGroupId(groupId);
            if (ObjectUtils.notEqual(list, null)) {
                return ResponseWrapper.success( 200, "查询成功", list );
            }
            return ResponseWrapper.error();
        } catch (Exception e) {
            log.error("数据字典查询失败{}", e.getMessage());
            return ResponseWrapper.error(500, "数据字典查询失败");
        }
    }

    /**
     * 查询某数据字典组下的数据字典树
     * @param groupId
     * @return
     */
    @ApiOperation(value = "查询某数据字典组下的数据字典树", notes = "查询某数据字典组下的数据字典树")
    @GetMapping(value = "/getDataTree/{groupId}")
    public ResponseWrapper getDataTree(@PathVariable("groupId") String groupId) {
        try {
            List<DataDictionaryVO> list = dataDictionaryService.getDataTreeByGroupId(groupId);
            /*Iterator<DataDictionaryVO> at = list.iterator();
            if (groupId.equals("AREA_CODE")){
            while (at.hasNext()){
                DataDictionaryVO dt = at.next();
                if (dt!=null&&dt.getId()!=120000){
                    at.remove();
                }
              }
            }*/
            return ResponseWrapper.success(200, "查询成功", list);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("数据字典树查询失败{}", e.getMessage());
            return ResponseWrapper.error(500, "数据字典树查询失败");
        }
    }

    /**
     * 查询所有数据字典组下的数据字典树
     * @return
     */
    @ApiOperation(value = "查询所有数据字典组下的数据字典树", notes = "查询所有数据字典组下的数据字典树")
    @GetMapping(value = "/getAllDataTree")
    public ResponseWrapper getAllDataTree() {
        try {
            List<Map> res = new ArrayList<>();
            List<DataDictionaryGroupVO> groupList = dataDictionaryGroupService.getAll(null);
            for(DataDictionaryGroupVO group : groupList) {
                if(StringUtils.isNotEmpty(group.getId())){
//                    DataDictionaryGroupVO groupInfo = dataDictionaryGroupService.findById(group.getId());
                    List<DataDictionaryVO> list = dataDictionaryService.getDataTreeByGroupId(group.getId());
                    if(list != null && list.size() > 0) {
                        for(DataDictionaryVO dd : list){
                            // 1字典
                            if(dd != null) {
                                dd.setType(1);
                            }
                        }
                    }
                    Map<String, Object> resMap = new HashMap<>();
                    resMap.put("dictCode",group.getId());
                    resMap.put("dictName",group.getGroupName());
                    resMap.put("pdictCode",group.getPid());
                    resMap.put("groupId",group.getId());
                    // 0字典组
                    resMap.put("type", 0);
                    resMap.put("subDataDictionarys",list);
                    res.add(resMap);
                }
            }
            return ResponseWrapper.success(200, "查询成功", res);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("数据字典树查询失败{}", e.getMessage());
            return ResponseWrapper.error(500, "数据字典树查询失败");
        }
    }

    /**
     * 查询某数据字典下的数据字典树
     * @return ResponseWrapper
     * @author 杨祖榕
     * @date 2018年11月10日 下午16:15:23
     * @version v1.0
     */
    @ApiOperation(value = "查询某数据字典下的数据字典树", notes = "查询某数据字典下的数据字典树")
    @GetMapping(value = "/getDataTree/{groupId}/{dictCode}")
    public ResponseWrapper getDataTree(@PathVariable("groupId") String groupId, @PathVariable("dictCode") String dictCode) {
        try {
            List<DataDictionaryVO> list = dataDictionaryService.getDataTree(groupId, dictCode);
            return ResponseWrapper.success( 200, "查询成功", list );
        } catch (Exception e) {
            log.error("数据字典树查询失败{}", e.getMessage());
            return ResponseWrapper.error(500, "数据字典树查询失败");
        }
    }

    @ApiOperation(value = "获取用户所在行政区划的行政区划树", notes = "参数：无")
    @GetMapping(value = "getUserAreaCodeTree")
    public ResponseWrapper getUserAreaCodeTree(){
        String areaCode = CurrentUtil.getAreaCode();//当前用户的行政区划code
        String groupId = "AREA_CODE";//行政区划的GroupId
        try {
            DataDictionaryVO result = dataDictionaryService.getDataTreeByDictCode(groupId, areaCode);
            return ResponseWrapper.success( 200, "查询成功", result );
        } catch (Exception e) {
            log.error("获取用户所在行政区划的行政区划树 查询失败{}", e.getMessage());
            return ResponseWrapper.error(500, "获取用户所在行政区划的行政区划树 查询失败");
        }
    }


    @ApiOperation(value = "获取用户所在行政区划的父节点行政区划树", notes = "参数：无")
    @GetMapping(value = "getUserAndAreaCodeTree")
    public ResponseWrapper getUserAndAreaCodeTree(){
        String areaCode = CurrentUtil.getAreaCode();//当前用户的行政区划code
        String groupId = "AREA_CODE";//行政区划的GroupId
        try {
             DataDictionaryVO result = dataDictionaryService.getDictCode(groupId, areaCode);
        	return ResponseWrapper.success( 200, "查询成功", result );
        } catch (Exception e) {
            log.error("获取用户所在行政区划的父节点行政区划树 查询失败{}", e.getMessage());
            return ResponseWrapper.error(500, "获取用户所在行政区划的父节点行政区划树 查询失败");
        }
    }


    /**
     * 查询某数据字典组下的数据字典树
     * @param groupId
     * @return
     */
    @ApiOperation(value = "查询某数据字典组下的数据字典树", notes = "查询某数据字典组下的数据字典树")
    @GetMapping(value = "/getDataGroupId/{groupId}")
    public ResponseWrapper getDataGroupId(@PathVariable("groupId") String groupId) {
        try {
            List<DataDictionaryVO> list = dataDictionaryService.getDataGroupId(groupId);
            return ResponseWrapper.success( 200, "查询成功", list );
        } catch (Exception e) {
            e.printStackTrace();
            log.error("数据字典树查询失败{}", e.getMessage());
            return ResponseWrapper.error(500, "数据字典树查询失败");
        }
    }

    /**
     * @Description: 新版查询某数据字典下的数据字典树
     * @date 2019年1月29日,下午5:40:44
     * @author huangshuai
     * @version v1.0
     * @param groupId
     * @return
     */
    @ApiOperation(value = "新版查询某数据字典下的数据字典树非必填", notes = "參數 : 父id , 组id")
    @GetMapping(value = "/getDataGroup")
    public ResponseWrapper getDataGroup(String groupId,String pdictCode) {
        try {
            List<DataDictionaryVO> list = dataDictionaryService.getSubDataDictionarys(groupId ,pdictCode);
            return ResponseWrapper.success( 200, "查询成功", list );
        } catch (Exception e) {
            e.printStackTrace();
            log.error("数据字典树查询失败{}", e.getMessage());
            return ResponseWrapper.error(500, "数据字典树查询失败");
        }
    }

    /**
     * @Description: 新版查询某数据字典下的数据字典树
     * @date 2019年1月29日,下午5:40:44
     * @author huangshuai
     * @version v1.0
     * @param groupId
     * @return
     */
    @ApiOperation(value = "新版查询某数据字典下的数据字典树", notes = "參數 : 父id , 组id")
    @GetMapping(value = "/getDataGroup/{groupId}/{pdictCode}")
    public ResponseWrapper getSubDataDictionarys(@PathVariable("groupId") String groupId , @PathVariable("pdictCode") String pdictCode) {
    	try {
    		List<DataDictionaryVO> list = dataDictionaryService.getSubDataDictionarys(groupId ,pdictCode);
    		return ResponseWrapper.success( 200, "查询成功", list );
    	} catch (Exception e) {
    		e.printStackTrace();
    		log.error("数据字典树查询失败{}", e.getMessage());
    		return ResponseWrapper.error(500, "数据字典树查询失败");
    	}
    }


//    @ApiOperation(value = "将数据字典 全部存入Redis", notes = "将数据字典 全部存入Redis")
//    @GetMapping(value = "/totalDataDictionary2Redis")
//    public ResponseWrapper totalDataDictionary2Redis() {
//        try {
////            dataDictionaryService.dataDictionarys2Redis();
//            dataDictionaryService.clearRedisDataDictionary();
//            return ResponseWrapper.success( 200, "将数据字典 全部存入Redis 操作成功!!!!!!!" );
//        } catch (Exception e) {
//            e.printStackTrace();
//            return ResponseWrapper.error(500, "将数据字典 全部存入Redis 操作 失败???????");
//        }
//    }

    @ApiOperation(value = "新版根据数据字典组ID和数据字典名称查询数据字典", notes = "参数：数据字典组ID--groupId，数据字典名称--dictName")
    @GetMapping(value = "getDataDictionaryByGroupIdAndInDictName/{groupId}/{dictName}")
    public ResponseWrapper getDataDictionaryByGroupIdAndInDictName(@PathVariable("groupId") String groupId, @PathVariable("dictName") String dictName){
        try {
            String dictCode = dataDictionaryService.getDataDictionaryByGroupIdAndInDictName(groupId, dictName);
            return ResponseWrapper.success(200, "查询成功", dictCode);
        } catch (Exception e) {
            log.error("查询数据字典数据，发生异常", e.getMessage());
            return ResponseWrapper.error(500, "数据字典数据查询失败");
        }

    }

    @ApiOperation(value = "新版根据groupId和dictCode查询数据字典-获取父级数据", notes = "参数：groupId，dictCode")
    @GetMapping(value = "getDataParent/{groupId}/{dictCode}")
    public ResponseWrapper getDataParent(@PathVariable("groupId") String groupId
            , @PathVariable("dictCode") String dictCode){
        try {
            List<DataDictionaryDO> dataDictionaryList = dataDictionaryService.getDataParent(groupId, dictCode);
            return ResponseWrapper.success(200, "查询成功", dataDictionaryList);
        } catch (Exception e) {
            log.error("查询数据字典数据，发生异常", e.getMessage());
            return ResponseWrapper.error(500, "数据字典数据查询失败");
        }
    }

}
