package com.unis.ynls.dataDictionary.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.unis.ynls.dataDictionary.constant.CommonConstant;
import com.unis.ynls.dataDictionary.utils.jwt.JwtConstant;
import com.unis.ynls.dataDictionary.utils.jwt.JwtUtil;
import com.unis.ynls.dataDictionary.utils.vo.UserDTO;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.Serializable;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * 模仿单机服务中的Session功能
 */
@Component
public class SessionUtil {
 
    private final String MID_AT = "@";
    private final String PRE_REDIS_KEY_USERINF = "userinfo@";
 
    @Autowired
    private RedisUtil<Serializable> redisUtil;
 
 
    /**
     * 将用户信息放入Redis中(类似于将用户信息放入Session)
     * @param userDTO
     */
    public void setUserInfo(UserDTO userDTO) {
        String redisKey = PRE_REDIS_KEY_USERINF + userDTO.getUserCode();
        String redisValue = JSONObject.toJSONString(userDTO);
        redisUtil.setStr(redisKey, redisValue);
    }
 
 
    /**
     * 获取当前登录用户信息
     * @return
     */
    public UserDTO getUserInfo() {
        UserDTO userDTO = null;
 
        String token = getToken();
 
        if(StringUtils.isNotEmpty(token)) {
            String userId = JwtUtil.getUserId(token);
            String redisKey = PRE_REDIS_KEY_USERINF + userId;
            //
            String userInfo = redisUtil.getStr(redisKey);
            userDTO = JSON.parseObject(userInfo, UserDTO.class);
        }
 
        return userDTO;
    }
 
    /**
     * 生成token，类似于SessionId
     * @param
     * @return
     */
    public String createToken(UserDTO userDTO) {
        String token = JwtUtil.createJWT(JwtConstant.JWT_ID, userDTO.getUserCode(), userDTO.getUserLoginName(), JwtConstant.ISS_USER, "", JwtConstant.JWT_TTL);
 
//        //@TODO 后期删除
//        token = CommonConstant.TEMP_TOKEN;
 
        return token;
    }
 
    public String getToken() {
        HttpServletRequest request = getRequest();
        String token = request.getHeader(JwtConstant.TOKEN);
 
//        //@TODO 后期删除
//        token = CommonConstant.TEMP_TOKEN;
 
        return token;
    }
 
    /**
     * //设置token的过期时间
     * @param token
     */
    public void setTokenExpiration(String token) {
        redisUtil.setForTime(JwtUtil.getJwtId(token), token, JwtUtil.CREAT_TIME + JwtConstant.JWT_TTL/2, TimeUnit.DAYS);
    }
 
 
    public void removeToken() {
        String token = getToken();
        if(StringUtils.isNotEmpty(token)) {
            redisUtil.delete(JwtUtil.getJwtId(token));
        }
    }
 
    public void removeUserInfo() {
        String token = getToken();
        if(StringUtils.isNotEmpty(token)) {
            String userId = JwtUtil.getUserId(token);
            String redisKey = PRE_REDIS_KEY_USERINF + userId;
            redisUtil.delete(redisKey);
        }
    }
 
    public void removeUserRights() {
        String token = getToken();
        if(StringUtils.isNotEmpty(token)) {
            String userId = JwtUtil.getUserId(token);
            String redisKey = CommonConstant.USER_RIGHTS_REDIS_KEY_PRE + MID_AT + userId;
            redisUtil.delete(redisKey);
            redisKey = CommonConstant.ADMIN_RIGHTS_REDIS_KEY_PRE + MID_AT + userId;
            redisUtil.delete(redisKey);
        }
    }
 
 
 
    /**
     * 获取用户ID
     * @return
     */
    public String getUserId(){
        return getUserInfo().getUserCode();
    }
 
    /**
     * 获取用户账号
     * @return
     */
    public String getAccount() {
        return getUserInfo()==null?null:getUserInfo().getUserLoginName();
    }
 
    /**
     * 获取用户姓名
     * @return
     */
    public String getUserName() {
        return getUserInfo().getUserName();
    }
 
    /**
     * 获取用户单位ID
     * @return
     */
    public String getOrgId() {
        return getUserInfo().getOrgCode();
    }
 
    /**
     * 获取用户单位名称
     * @return
     */
    public String getOrgName() {
        return getUserInfo().getOrgName();
    }
 
    /**
     * 获取用户部门ID
     * @return
     */
    public String getDeptId() {
        return getUserInfo().getDeptCode();
    }
 
    /**
     * 获取用户部门名称
     * @return
     */
    public String getDeptName() {
        return getUserInfo().getDeptName();
    }
 
    /**
     * 获取用户行政区划
     * @return
     */
    public String getAreaCode(){
        return getUserInfo().getCountyCode();
    }
 
    /**
     * 获取密码
     * @return
     */
    public String getPassWord(){
        return getUserInfo().getUserPassword();
    }
    /**
     * 获取角色
     * @return
     */
    public List<String> getUserRole(){
    	return getUserInfo().getUserRole();
    }
 
    /**
     * 获取用户类型 用户类型，在数据字典维护，1：粮食局，2：企业用户
     * @return
     */
    public String getUserType(){
        return getUserInfo().getUserType();
    }
 
    /**
     * 获取省份编码
     * @return
     */
   /* public String getProvinceCode(){
        return getUserInfo().getProvinceCode();
    }*/

//    /**
//     * 获取用户行政区划编码
//     * @return
//     */
//    public String getAreaCode() {
//        return getUserInfo().getAreaCode();
//    }
//
//    /**
//     * 获取用户登陆密码
//     * @return
//     */
//    public String getPassWord() {
//        return getUserInfo().getPassword();
//    }
//
//    /**
//     * 获取企业用户ID
//     * @return
//     */
//    public Long getCompanyId() {
//        return getUserInfo().getQyUserId();
//    }
//
//    /**
//     *  获取用户数据范围
//     * @return
//     */
//    public Integer getRange(){
//        return getUserInfo().getRange();
//    }
//
//    /**
//     * 获取用户流程节点等级
//     * @return
//     */
//    public Integer getUserNodeLevel(){
//        return getUserInfo().getUserNodeLevel();
//    }
//
//    /**
//     * @return
//     * @description:1、粮食局用户 2、企业用户
//     */
//    public Integer getUserType() {
//        return getUserInfo().getSign();
//    }
//
//    /**
//     * 获取企业基本信息
//     *
//     * @return
//     */
//    public QyUserTO getCompanyInfo() {
//        QyUserTO qyUserTO = null;
//        HttpServletRequest request = getRequest();
//        String jwt = request.getHeader(JwtConstant.JWT_KEY);
//        StringBuffer stringBuffer = new StringBuffer("company_baseinfo");
//        stringBuffer.append("@");
//        stringBuffer.append(JwtUtil.parseJWT(jwt).get(JwtUtil.USER_ID, Long.class));
//        String redisKey = stringBuffer.toString();
//        String companyInfo = redisUtil.getStr(redisKey);
//        qyUserTO = JSON.parseObject(companyInfo.toString(), QyUserTO.class);
//        return qyUserTO;
//    }
//
//    /**
//     * 获取企业行政区划级别
//     *
//     * @return
//     */
//    public int getCompanyAreaLevel() {
//        String userAreaCode = getCompanyInfo().getAreaCode();
//        StringUtils.substring(userAreaCode, 2, 6);
//        if (StringUtils.substring(userAreaCode, 2, 6).equals("0000")) {
//            //省级
//            return 1;
//        } else if (StringUtils.substring(userAreaCode, 4, 6).equals("00")) {
//            //市级
//            return 2;
//        } else {
//            //县级
//            return 3;
//        }
//    }
//
//    /**
//     * 获取用户行政区划等级
//     * @return
//     */
//    public int getUserAreaLevel() {
//        String userAreaCode = getUserInfo().getAreaCode();
//        StringUtils.substring(userAreaCode, 2, 6);
//        if (StringUtils.substring(userAreaCode, 2, 6).equals("0000")) {
//            //省级
//            return 1;
//        } else if (StringUtils.substring(userAreaCode, 4, 6).equals("00")) {
//            //市级
//            return 2;
//        } else {
//            //县级
//            return 3;
//        }
//    }
 
 
    /**
     * @return
     * @Description: 获取request
     * @date 2018年11月27日, 下午4:15:29
     * @author tiechanglong
     * @version 1.2.0
     */
    public static HttpServletRequest getRequest() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        return request;
    }
 
    /**
     * @return
     * @Description: 获取response
     * @date 2018年11月27日, 下午4:15:26
     * @author tiechanglong
     * @version 1.2.0
     */
    public static HttpServletResponse getResponse() {
        return  ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getResponse();
    }
}